#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const crypto = require("node:crypto");
const fs = require("node:fs");
const os = require("node:os");
const path = require("node:path");
const minimist = require("minimist");
const rimraf_1 = require("rimraf");
const vscode_uri_1 = require("vscode-uri");
const helpers_1 = require("../lib/helpers");
const markdown_1 = require("../lib/markdown");
async function typeCheckFiles(tempDir, filenameMapping, filenames, typings) {
    const tscExec = path.join(require.resolve('typescript'), '..', '..', 'bin', 'tsc');
    const options = ['--noEmit', '--pretty', '--moduleDetection', 'force'];
    if (filenames.find((filename) => filename.endsWith('.js'))) {
        options.push('--checkJs');
    }
    const args = [tscExec, ...options, ...typings, ...filenames];
    const { status, stderr, stdout } = await (0, helpers_1.spawnAsync)(process.execPath, args);
    if (stderr) {
        throw new Error(stderr);
    }
    // Replace the temp file paths with the original source filename
    let correctedOutput = stdout.replace(new RegExp(`${path.relative(process.cwd(), tempDir)}${path.sep}`.replace(/\\/g, path.posix.sep), 'g'), '');
    // Strip any @ts-expect-error comments we added
    correctedOutput = correctedOutput.replace(/ \/\/ @ts-expect-error/g, '');
    if (correctedOutput.trim()) {
        for (const [filename, originalFilename] of filenameMapping.entries()) {
            correctedOutput = correctedOutput.replace(new RegExp(path.basename(filename), 'g'), originalFilename);
        }
        console.log(correctedOutput);
    }
    return status;
}
function parseDirectives(directive, value) {
    return (0, helpers_1.findCurlyBracedDirectives)(directive, value)
        .map((parsed) => parsed.match(/^([^:\r\n\t\f\v ]+):\s?(.+)$/))
        .filter((parsed) => parsed !== null);
}
async function main(workspaceRoot, globs, { config = undefined, ignoreGlobs = [] }) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j;
    const workspace = new markdown_1.DocsWorkspace(workspaceRoot, globs, ignoreGlobs);
    const tempDir = fs.mkdtempSync(path.join(os.tmpdir(), 'lint-roller-ts-check-'));
    try {
        const filenames = [];
        const originalFilenames = new Map();
        const isolateFilenames = new Set();
        let ambientModules = '';
        let errors = false;
        for (const document of await workspace.getAllMarkdownDocuments()) {
            const uri = vscode_uri_1.URI.parse(document.uri);
            const filepath = workspace.getWorkspaceRelativePath(uri);
            const codeBlocks = (await (0, markdown_1.getCodeBlocks)(document.getText())).filter((code) => code.lang && ['javascript', 'js', 'typescript', 'ts'].includes(code.lang.toLowerCase()));
            for (const codeBlock of codeBlocks) {
                const isTypeScript = codeBlock.lang && ['typescript', 'ts'].includes(codeBlock.lang.toLowerCase());
                const line = codeBlock.position.start.line;
                const indent = codeBlock.position.start.column - 1;
                const tsNoCheck = (_a = codeBlock.meta) === null || _a === void 0 ? void 0 : _a.split(' ').includes('@ts-nocheck');
                const tsNoIsolate = (_b = codeBlock.meta) === null || _b === void 0 ? void 0 : _b.split(' ').includes('@ts-noisolate');
                const tsExpectErrorLines = (_d = (_c = codeBlock.meta) === null || _c === void 0 ? void 0 : _c.match(/\B@ts-expect-error=\[([\d,]*)\]\B/)) === null || _d === void 0 ? void 0 : _d[1].split(',').map((line) => parseInt(line));
                const tsTypeLines = codeBlock.meta ? parseDirectives('@ts-type', codeBlock.meta) : [];
                const tsWindowTypeLines = codeBlock.meta
                    ? parseDirectives('@ts-window-type', codeBlock.meta)
                    : [];
                if (tsNoCheck && (tsExpectErrorLines || tsTypeLines.length || tsWindowTypeLines.length)) {
                    console.log(`${filepath}:${line}:${indent + 1}: Code block has both @ts-nocheck and @ts-expect-error/@ts-type/@ts-window-type, they conflict`);
                    errors = true;
                    continue;
                }
                // Skip blocks with @ts-nocheck in their info string
                if (tsNoCheck) {
                    continue;
                }
                // Skip empty code blocks
                if (!codeBlock.value.trim()) {
                    continue;
                }
                const codeLines = codeBlock.value.split('\n');
                let insertedInitialLine = false;
                let types = '';
                let windowTypes = '';
                const insertComment = (comment, line) => {
                    // Inserting additional lines will make the tsc output
                    // incorrect which would be a pain to manually adjust,
                    // and there is no @ts-expect-error-line, so tack the
                    // comment on to the end of the previous line - looks
                    // ugly but we never have to see it since it's in a temp
                    // file. The first line of the file is an edge case where
                    // an insertion is necessary, so take that into account
                    if (line === 1) {
                        codeLines.unshift(comment);
                        insertedInitialLine = true;
                    }
                    else {
                        const offset = insertedInitialLine ? 1 : 2;
                        const codeLine = codeLines[line - offset];
                        // If the line is already a comment, fully replace it,
                        // otherwise tsc won't pick up the inserted comment
                        if (codeLine.match(/^\s*\/\/\s/)) {
                            codeLines[line - offset] = comment;
                        }
                        else {
                            codeLines[line - offset] = `${codeLine} ${comment}`;
                        }
                    }
                };
                // Blocks can have @ts-expect-error=[1,10,50] in their info string
                // (1-based lines) to insert an "// @ts-expect-error" comment before
                // specified lines, in order to ignore specific lines (like
                // requires of extra modules) without skipping the whole block
                for (const line of tsExpectErrorLines !== null && tsExpectErrorLines !== void 0 ? tsExpectErrorLines : []) {
                    insertComment('// @ts-expect-error', line);
                }
                // Indent the lines if necessary so that tsc output is accurate
                const code = (0, helpers_1.wrapOrphanObjectInParens)(codeLines
                    .map((line) => (line.length ? line.padStart(line.length + indent) : line))
                    .join('\n'));
                // If there are no require() or import lines, insert a default set of
                // imports so that most snippets will have what they need.
                // This isn't foolproof and might cause name conflicts
                const imports = codeBlock.value.match(/^\s*(?:import .* from )|(?:.* = require())/m)
                    ? ''
                    : ((_g = (_f = (_e = config === null || config === void 0 ? void 0 : config['markdown-ts-check']) === null || _e === void 0 ? void 0 : _e.defaultImports) === null || _f === void 0 ? void 0 : _f.join(';')) !== null && _g !== void 0 ? _g : '') + ';';
                // Insert the necessary number of blank lines so that the line
                // numbers in output from tsc is accurate to the original file
                const blankLines = '\n'.repeat(Math.max(0, insertedInitialLine ? line - 3 : line - 2));
                // Filename is unique since it is the name of the original Markdown
                // file, with the starting line number of the codeblock appended
                const filename = path.join(tempDir, `${filepath
                    .replace(new RegExp(path.sep.replace(/\\/g, '\\\\'), 'g'), '-')
                    .replace(/\./g, '-')}-${line}.${isTypeScript ? 'ts' : 'js'}`);
                // Blocks can have @ts-type={name:type} in their info
                // string to declare a global variable for a block
                if (tsTypeLines.length) {
                    // To support this feature, generate a random name for a
                    // module, generate an ambient module declaration for the
                    // module, and then import the global variables we're
                    // defining from that module name - there's no code on
                    // disk for it, only an ambient module declaration which
                    // tsc will use to type the phantom variables being imported
                    const moduleName = crypto.randomBytes(16).toString('hex');
                    const extraTypes = tsTypeLines
                        .map((type) => `  export var ${type[1]}: ${type[2]};`)
                        .join('\n');
                    ambientModules += `declare module "${moduleName}" {\n${extraTypes}\n}\n\n`;
                    types = `const {${tsTypeLines
                        .map((type) => type[1])
                        .join(',')}} = require('${moduleName}')`;
                }
                // Blocks can have @ts-window-type={name:type} in their
                // info string to extend the Window object for a block
                if (!tsNoIsolate && tsWindowTypeLines.length) {
                    const extraTypes = tsWindowTypeLines
                        .map((type) => `    ${type[1]}: ${type[2]};`)
                        .join('\n');
                    // Needs an export {} at the end to make TypeScript happy
                    windowTypes = `declare global {\n  interface Window {\n${extraTypes}\n  }\n}\n\nexport {};\n\n`;
                    fs.writeFileSync(filename.replace(/.[jt]s$/, '-window.d.ts'), windowTypes);
                    isolateFilenames.add(filename);
                }
                else if (!tsNoIsolate && code.match(/^\s*declare global /m)) {
                    isolateFilenames.add(filename);
                }
                else {
                    filenames.push(filename);
                }
                fs.writeFileSync(filename, `// @ts-check\n${imports}\n${blankLines}${code}\n${types}`);
                originalFilenames.set(filename, filepath);
            }
        }
        const ambientTypings = path.join(tempDir, 'ambient.d.ts');
        fs.writeFileSync(ambientTypings, ambientModules);
        const typings = [ambientTypings];
        // Copy over the typings so that a relative path can be used
        for (const typing of (_j = (_h = config === null || config === void 0 ? void 0 : config['markdown-ts-check']) === null || _h === void 0 ? void 0 : _h.typings) !== null && _j !== void 0 ? _j : []) {
            const tempPath = path.join(tempDir, path.basename(typing));
            fs.copyFileSync(path.join(path.resolve(workspaceRoot), typing), tempPath);
            typings.push(tempPath);
        }
        // Files for code blocks with window type directives or 'declare global' need
        // to be processed separately since window types are by nature global, and
        // they would bleed between blocks otherwise, which can cause problems
        for (const filename of isolateFilenames) {
            const filenames = [filename];
            const windowTypesFilename = filename.replace(/.[jt]s$/, '-window.d.ts');
            try {
                fs.statSync(windowTypesFilename);
                filenames.unshift(windowTypesFilename);
            }
            catch (_k) { }
            const status = await typeCheckFiles(tempDir, originalFilenames, filenames, typings);
            errors = errors || status !== 0;
        }
        // For the rest of the files, run them all at once so it doesn't take forever
        for (const chunk of (0, helpers_1.chunkFilenames)(filenames)) {
            const status = await typeCheckFiles(tempDir, originalFilenames, chunk, typings);
            errors = errors || status !== 0;
        }
        return errors;
    }
    finally {
        await (0, rimraf_1.rimraf)(tempDir);
    }
}
function parseCommandLine() {
    const showUsage = (arg) => {
        if (!arg || arg.startsWith('-')) {
            console.log('Usage: lint-roller-markdown-ts-check [--root <dir>] <globs> [-h|--help]' +
                '[--ignore <globs>] [--ignore-path <path>] [--config <path>]');
            process.exit(1);
        }
        return true;
    };
    const opts = minimist(process.argv.slice(2), {
        boolean: ['help'],
        string: ['config', 'root', 'ignore', 'ignore-path'],
        unknown: showUsage,
    });
    if (opts.help || !opts._.length)
        showUsage();
    return opts;
}
if (require.main === module) {
    const opts = parseCommandLine();
    if (!opts.root) {
        opts.root = '.';
    }
    if (opts.ignore) {
        opts.ignore = Array.isArray(opts.ignore) ? opts.ignore : [opts.ignore];
    }
    else {
        opts.ignore = [];
    }
    if (opts['ignore-path']) {
        const ignores = fs.readFileSync(path.resolve(opts['ignore-path']), { encoding: 'utf-8' });
        for (const ignore of ignores.split('\n')) {
            opts.ignore.push(ignore.trimEnd());
        }
    }
    const config = (0, helpers_1.loadConfig)(opts.config ? path.resolve(opts.config) : path.resolve('.lint-roller.json'));
    main(path.resolve(process.cwd(), opts.root), opts._, {
        config,
        ignoreGlobs: opts.ignore,
    })
        .then((errors) => {
        if (errors)
            process.exit(1);
    })
        .catch((error) => {
        console.error(error);
        process.exit(1);
    });
}
//# sourceMappingURL=lint-markdown-ts-check.js.map