import { SemVer } from 'semver';
export { SemVer };
import { Paths } from './paths';
export declare type SemOrStr = SemVer | string;
/**
 * Interface for an object that manages a list of Electron releases.
 *
 * See {@link BaseVersions} for testing situations.
 * See {@link ElectronVersions} for production.
 */
export interface Versions {
    /** Semver-Major numbers of branches that only have prereleases */
    readonly prereleaseMajors: number[];
    /** Semver-Major numbers of branches that have supported stable releases */
    readonly supportedMajors: number[];
    /** Semver-Major numbers of branches that are no longer supported */
    readonly obsoleteMajors: number[];
    /** The latest release (by version, not by date) */
    readonly latest: SemVer | undefined;
    /** The latest stable (by version, not by date) */
    readonly latestStable: SemVer | undefined;
    /** Full list of all known Electron releases, Sorted in branch order. */
    readonly versions: SemVer[];
    /** @returns true iff `version` is a release that this object knows about */
    isVersion(version: SemOrStr): boolean;
    /** @returns all versions matching that major number. Sorted in branch order. */
    inMajor(major: number): SemVer[];
    /** @returns all versions in a range, inclusive. Sorted in branch order. */
    inRange(a: SemOrStr, b: SemOrStr): SemVer[];
}
export declare function compareVersions(a: SemVer, b: SemVer): number;
/**
 * Implementation of {@link Versions} that does everything except self-populate.
 * It needs to be fed version info in its constructor.
 *
 * In production, use subclass '{@link ElectronVersions}'. This base class is
 * useful in testing because it's an easy way to inject fake test data into a
 * real Versions object.
 */
export declare class BaseVersions implements Versions {
    private readonly map;
    protected setVersions(val: unknown): void;
    constructor(versions: unknown);
    get prereleaseMajors(): number[];
    get stableMajors(): number[];
    get supportedMajors(): number[];
    get obsoleteMajors(): number[];
    get versions(): SemVer[];
    get latest(): SemVer | undefined;
    get latestStable(): SemVer | undefined;
    isVersion(ver: SemOrStr): boolean;
    inMajor(major: number): SemVer[];
    inRange(a: SemOrStr, b: SemOrStr): SemVer[];
}
/**
 * Implementation of Versions that self-populates from release information at
 * https://releases.electronjs.org/releases.json .
 *
 * This is generally what to use in production.
 */
export declare class ElectronVersions extends BaseVersions {
    private readonly versionsCache;
    private mtimeMs;
    private constructor();
    private static fetchVersions;
    private static isCacheFresh;
    static create(paths?: Partial<Paths>): Promise<ElectronVersions>;
    fetch(): Promise<void>;
    private keepFresh;
    get prereleaseMajors(): number[];
    get stableMajors(): number[];
    get supportedMajors(): number[];
    get obsoleteMajors(): number[];
    get versions(): SemVer[];
    get latest(): SemVer | undefined;
    get latestStable(): SemVer | undefined;
    isVersion(ver: SemOrStr): boolean;
    inMajor(major: number): SemVer[];
    inRange(a: SemOrStr, b: SemOrStr): SemVer[];
}
